/**************************************************************************************
 PK Programmierpraktikum SS 99
 Gruppe: Noisternig M., Obermair C., Schmidbauer J., Stieglbauer G.
 
 Projekt: Rug Warrior Communication
 
 Datei:
   debug.c
 
 Typ:
   Bibliothek
 
 Beschreibung:   
   Funktionen die ein einfaches Debugging (inklusive Zeichenausgabe auf
   dem LCD-Display) ermoeglichen.
   
   void init_LCD(void);
   	Initialisiert das LCD-Display und loescht dessen aktuellen Inhalt.
   
   void printstring(char *s);
   	Gibt einen beliebigen String auf dem LCD-Display aus.
   	Eingabe: char *s (Pointer auf den auszugebenden String)
   
   void printnum(int i, int b);
   	Gibt eine Ganzzahl i zur Basis b auf dem LCD-Display aus.
   	Eingabe: int i (Auszugebender Zahlenwert)
   		 int b (Basis; Gueltige Werte sind 2 bis 10, sowie 16
   		 	fuer hexadezimale Darstellung)
   		 	
   void num2LED(byte i);
   	Gibt eine 4-Bit Zahl auf den LEDs des Rug Warriors im Binaerformat
   	aus
   	Eingabe: int i (Anzuzeigende Zahl; die obersten 4-Bits werden
   			abgeschnitten.)
   
   void trap(void);
   	Simuliert einen Software-Trap. Das laufende Programm wird solange
   	angehalten, bis ein Bumper gedrueckt und wieder losgelassen wurde.
   	Zusaetzliche Statusanzeigen werden auf den LEDs ausgegeben.
      
  Plattform: 
   Rug Warrior (Motorola 68HC11)

*****************************************************************************/

#pragma long_branch

#include <os.h>
#include <string.h>
#include <reg6811.h>

#define LED_MASK 0x3c
#define LCD_ROWS 16

extern void LCDStart(unsigned int);
extern void LCDCopy(void);
void init_LCD(void);
void printstring(char *);
void printnum(int, int);
void num2LED(byte);
void trap(void);

void init_LCD(void)
{
    word init_seq[] = {0x0038, 0x000c, 0x0001, 0x0006, 0x008f, 0};
    byte i;
    
    LCDCopy();				/* Ausfuehren der Initsequenz */
    for(i = 0; init_seq[i]; i++)	/* am LCD-Display */
        LCDStart(init_seq[i]);
}

void printstring(char *s)
{
    int i;

    LCDStart(0x0002);
    for(i = 0; *s != '\0'; s++, i++)
    {
        if (i == LCD_ROWS)
            while(i++ < LCD_ROWS*2+8)           /* Zeilenwechsel */
                LCDStart(0x0200 + ' ');          
        LCDStart(0x0200 + (unsigned int)(*s));  /* Ausgabe der Zeichen */
    }
}

void printnum(int i, int base)
{
    int s[sizeof(int) * 8];
    int j;

    LCDStart(0x0002);

    if (i < 0)				/* Vorzeichenbehandlung */
    {
        LCDStart(0x0200 + '-');
        i = -i;
    }

    if (base == 16)			/* Sonderfall: Hexadezimal */
    {
        LCDStart(0x0200 + '0');
        LCDStart(0x0200 + 'x');
    }

    if (i == 0)				/* Sonderfall: Wert 0 */
    {
        LCDStart(0x0200 + '0');
        return;
    }

    for(j = 0; i != 0; j++, i /= base)		/* Berechnung der einzelnen Ziffern */
        s[j] = i - ((int)(i / base)) * base;	/* zur jeweiligen Basis */

    for(j--; j >= 0; j--)			/* Ausgabe der Ziffern */
        LCDStart(0x0200 + (s[j] > 10 ? s[j] + 'A' - 10 : s[j] + '0'));
}

void num2LED(byte i)
{
    DDRD |= LED_MASK;
    i <<= 2;
    i &= LED_MASK;
    PORTD = (PORTD &= ~LED_MASK) | i ;		/* Ausgabe der Zahl im Binaerformat */
}

void trap(void)
{
    byte led, status;

    DDRD |= LED_MASK;
    led = (LED_MASK & PORTD) >> 2;
    
    
    num2LED(1);			/* Programm im Wartezustand (Bumper nicht gedrueckt) */

    OPTION &= 0x80;		/* Abfrage, ob Bumper gedrueckt */
    do
    {
        ADCTL = 0x13;
        while(ADCTL & 0x80 == 0);
    } while (ADR4 < 0x10);

    num2LED(2);			/* Programm im Wartezustand (Bumper gedrueckt) */

    while (ADR4 >= 0x10)	/* Abfrage, ob Bumper losgelassen */
    {
        ADCTL = 0x13;
        while(ADCTL & 0x80 == 0);
    }

    num2LED(led);		/* Wiederherstellen des urspruenglichen Zustandes */
    				/* und Fortsetzung des Programmes */

}   
            

