/**************************************************************************************
 PK Programmierpraktikum SS 99
 Gruppe: Noisternig M., Obermair C., Schmidbauer J., Stieglbauer G.
 
 Projekt: Rug Warrior Communication
 
 Datei:
    rug_lib.c
 
 Typ:
   Bibliothek
 
 Beschreibung:   
   Funktionen zur Ansteuerung der Peripherie des Rug Warriors, eine
   Fast Fourier Transformation, sowie einige Tonsequenzen.
   
   void fft_inittab(void);
   	Laedt eine Sinus-Tabelle zur Beschleunigung der FFT in den Speicher.
   	
   void fft(void);
   	Fast Fourier Transformation
   	Eingabe: Fixe Speicheradresse
   
   void note(int freq, int len);
   	Abgabe eines Tons auf dem Piezo-Lautsprecher
   	Eingabe: int freq (Frequenz des Tons)
   		 int len  (Laenge des Tons)
   		 
   void tone(int freq, int len);
   	Initialisation der Interrupt-Routine und Anpassen der Uebergabewerte
   	Eingabe: wie bei note
   	
   void irq_tone(void);
   	Interrupt-Routine zur Tonabgabe.   
   	
   void closeencounters(void);
   	Tonsequenz "Close Encounters"
   	
   void cof(void);
   	Tonsequenz "Chariots of Fire"
   	
   void init_pwm(void);
   	Initialisierung der Puls-Weiten-Modulations-Einheit auf dem 68HC11
   	fuer die Motoransteuerung
   	
   void motor(byte index, schar vel);
   	Ansteuerung der einzelnen Motoren.
   	Eingabe: byte index (Auswahl des Motors:
   			    	0...Linker Motor
   			    	1...Rechter Motor)
   		 schar vel  (Geschwindigkeit und Rotationsrichtung des Motors)
   		 
   void drive(schar trans_vel, schar rot_vel)
   	Bestimmung der Fahrrichtung und Geschwindigkeit des Rug Warriors
   	Eingabe: schar trans_vel (Geschwindigkeit des Rug Warriors)
   		 schar rot_vel   (Rotation des Rug Warriors)

      
  Plattform: 
   Rug Warrior (Motorola 68HC11)

***********************************************************************************/

#pragma long_branch
#include <os.h>
#include <debug.h>
#include <reg6811.h>
#include <rug_lib.h>


void fft_inittab()
{
  int i;
  fft_e_rp[0]=     0; fft_e_ip[0]=     0;	/* Sinustabelle */
  fft_e_rp[1]= 32000; fft_e_ip[1]=     0;
  fft_e_rp[2]= 31385; fft_e_ip[2]=  6242;
  fft_e_rp[3]= 29564; fft_e_ip[3]= 12245;
  fft_e_rp[4]= 26607; fft_e_ip[4]= 17778;
  fft_e_rp[5]= 22627; fft_e_ip[5]= 22627;
  fft_e_rp[6]= 17778; fft_e_ip[6]= 26607;
  fft_e_rp[7]= 12245; fft_e_ip[7]= 29564;
  fft_e_rp[8]=  6242; fft_e_ip[8]= 31385;
  fft_e_rp[9]=     0; fft_e_ip[9]= 32000;
  fft_e_rp[10]= -6242; fft_e_ip[10]= 31385;
  fft_e_rp[11]=-12245; fft_e_ip[11]= 29564;
  fft_e_rp[12]=-17778; fft_e_ip[12]= 26607;
  fft_e_rp[13]=-22627; fft_e_ip[13]= 22627;
  fft_e_rp[14]=-26607; fft_e_ip[14]= 17778;
  fft_e_rp[15]=-29564; fft_e_ip[15]= 12245;
  fft_e_rp[16]=-31385; fft_e_ip[16]=  6242;
  fft_e_rp[17]=-32000; fft_e_ip[17]=     0;
  for(i=0;i<18;i++) {fft_e_rp[i]/=4000; fft_e_ip[i]/=4000;}
}


void fft()
{
  char i, j, k, l, m, index;
  int ber,bei,ftr,fti,im;
  m = fftsize / 2 ;
  for(l=1;l<=m;l*=2)
  {
    k = 0 ;
    i = 1 ;
    for(j=l;j<=m;k=j,j=k+l)
      for(;i<=j;i++)
      {
	im=i+m;

	ber=fft_e_rp[k+1];
	bei=fft_e_ip[k+1];
        ftr=(fft_z_r[i]-fft_z_r[im])/8;
        fti=(fft_z_i[i]-fft_z_i[im])/8;

	fft_w_r[i+k] = fft_z_r[i]+fft_z_r[im];
	fft_w_i[i+k] = fft_z_i[i]+fft_z_i[im];

	fft_w_r[i+j] = ber*ftr-bei*fti;
	fft_w_i[i+j] = ber*fti+bei*ftr;
      }
      for(index=1;index<=fftsize;index++)
      {
	fft_z_r[index]=fft_w_r[index];
	fft_z_i[index]=fft_w_i[index];
      }
  }

  for ( i = 1; i <= fftsize; i++ )
  {
    fft_z_r[i]/=35;
    fft_z_i[i]/=35;

    fft_z_r[i] = fft_z_r[i]*fft_z_r[i]+fft_z_i[i]*fft_z_i[i];
  }
}


/* Funktionen fuer Lautsprecher */

void irq_tone()
{ 
   
   length -= 3000/frequency;
   TOC5 += 65535U/(unsigned int)frequency*15;
   TFLG1 |= 0x08;

   #ifdef EXT_SPEAKER
   PORTA ^= 0x10;
   #endif
   
   if(length<0)
      { TCTL1&=~0x01;
        PORTA&=0x08;
        TMSK1 &= ~0x08;
      }
}

void note(int freq, int len)
{
   tone(freq, len);
   while(TCTL1 & 0x01);
}

void tone(int freq, int len)
{
  frequency = freq;
  length = len*6;
  
  os_disable();
  TCTL1 |= 0x01;
  TCTL1 &= ~0x02;
  TMSK2 &=~ 0x03;
  TMSK1 |= 0x08;
  os_set_irq(OS_IRQ_TOC5, irq_tone);
  os_enable();

  TOC5 += TCNT+65535U/(unsigned int)frequency*15;
}

void closeencounters()
{
	note(200, 400);
	note(300, 400);
	note(200, 400);
	note(150, 400);
	note(200, 800);
}

void cof()
{
  note(536,250);
  note(716,250); 
  note(796,250); 
  note(880,250);
  note(796,750);
  note(678,5 * 250);
  note(536,250);
  note(716,250); 
  note(796,250); 
  note(880,250);
  note(796,8 * 250);
  note(536,250);
  note(716,250); 
  note(796,250); 
  note(880,250);
  note(796,750);
  note(678,5 * 250);
  note(796,250);
  note(716,250); 
  note(678,250); 
  note(536,250);
  note(536,5 * 250);
  note(1072,250);
  note(1006,250); 
  note(880,250); 
  note(796,250);
  note(1006,500); 
  note(796,250);
  note(880,500); 
  note(716,250);
  note(796,500); 
  note(1072,250);
  note(1006,250); 
  note(880,250); 
  note(796,250);
  note(1006,8 * 250);
  note(1072,250);
  note(1006,250); 
  note(880,250); 
  note(796,250);
  note(1006,500); 
  note(796,250);
  note(880,500); 
  note(716,250);
  note(796,500); 
  note(796,250);
  note(716,250); 
  note(678,250); 
  note(536,250);
  note(536,7 * 250); 
}

/* Funktionen fuer Motor */

void init_pwm(void)
{
    TMSK2 = 0x00;
    DDRD = 0x30;
    OC1M = 0x60;
    TCTL1 |= OM2|OM3;
    TCTL1 &= ~(OL2|OL3);
    TOC2 = 0;
}

void motor(byte index, schar vel)
{
    schar avel;                   /* Absolute value of velocity */

    if (vel > 0)
    {
        PORTD |= (0x10<<index);  /* Forward rotation */
        avel = vel;
    }
    else
    {
        PORTD &= ~(0x10<<index); /* Backward rotation */
        avel = (- vel);
    }

    if (avel < 1)       /* If we are going real slow */
        OC1D &= ~(0x40>>index);  /* then just have OC1 turn off the motor */
    else
        OC1D |= (0x40>>index);    /* Otherwise let OC1 control the motors */

    if (avel > 99)      /* If we are gong real fast */
        avel = 99;        /* don't let the following multiply overflow */

    index == 0 ? (TOC2 = 655*(word)avel) : (TOC3 = 655*(word)avel);
}

/* Use drive to control motion of the robot.  A positive rot_vel makes the robot
   turn left. */

void drive(schar trans_vel, schar rot_vel)
{
    motor(0, trans_vel - rot_vel);
    motor(1, trans_vel + rot_vel);
}
